##############################
#  Parameters
##############################

param K; #number of respondents
param C; #number of competitors
param T; #number of coefficients prone to uncertainty


set RESPONDENT := 1..K;
set ATTRIBUTE; 
set ATTRIBUTE_LEVEL{ATTRIBUTE} ordered; 
set ORDINAL_ATTRIBUTES within ATTRIBUTE ordered;
set COMPETITOR :=1..C;


param utility{RESPONDENT, i in ATTRIBUTE, ATTRIBUTE_LEVEL[i]};
param utility_min_increase{RESPONDENT};
param utility_uncertainty{RESPONDENT, i in ATTRIBUTE, ATTRIBUTE_LEVEL[i]};
param weight{RESPONDENT} default 1/K;


param X_COMP{c in COMPETITOR, i in ATTRIBUTE, ATTRIBUTE_LEVEL[i]} default 0; #competition service design


##############################
#  Dependent parameters
##############################


param N = (sum{i in ORDINAL_ATTRIBUTES} card(ATTRIBUTE_LEVEL[i])) -card(ORDINAL_ATTRIBUTES); #number of constraints on the part-worths
set CONSTRAINT := 1..N;
param S{n in CONSTRAINT, i in ATTRIBUTE, ATTRIBUTE_LEVEL[i]} default 0; # matrice for the constraints on the part-worth (-1,0 or 1)




param BIGM = 
(max{k in RESPONDENT} utility_min_increase[k])
+(sum{i in ATTRIBUTE} 
	(
	max{k in RESPONDENT, j in ATTRIBUTE_LEVEL[i]} ( utility[k,i,j]+utility_uncertainty[k,i,j])
	)
)
-
(sum{i in ATTRIBUTE} 
	(
	min{k in RESPONDENT, j in ATTRIBUTE_LEVEL[i]} ( utility[k,i,j]-utility_uncertainty[k,i,j])
	)
)
;





##############################
#  Decision variables
##############################

var X{i in ATTRIBUTE, ATTRIBUTE_LEVEL[i]} binary; # service configuration



##############################
#  Dependent variables
##############################

var Y{RESPONDENT} binary; # preference for respondent

var BETA{COMPETITOR, RESPONDENT};
var Q{COMPETITOR, RESPONDENT} >= 0;
var P_P{c in COMPETITOR, k in RESPONDENT, i in ATTRIBUTE, j in ATTRIBUTE_LEVEL[i]} >= 0;
var P_M{c in COMPETITOR, k in RESPONDENT, i in ATTRIBUTE, j in ATTRIBUTE_LEVEL[i]} >= 0;
var R{CONSTRAINT, COMPETITOR, RESPONDENT}>= 0;




##############################
#  Model
##############################

maximize share:
(sum{k in RESPONDENT} weight[k]*Y[k])
#-0.00001*(sum {c in COMPETITOR, k in RESPONDENT} BETA[c,k])	
;


subject to share_of_choice{c in COMPETITOR, k in RESPONDENT}:
(sum{i in ATTRIBUTE, j in ATTRIBUTE_LEVEL[i]} utility[k,i,j]*(X[i,j]-X_COMP[c,i,j]))
-BETA[c,k]
>= 
utility_min_increase[k] + (Y[k] - 1)*BIGM
;


subject to normalization_x{i in ATTRIBUTE}:
sum{j in ATTRIBUTE_LEVEL[i]} X[i,j] = 1
;




subject to beta_equation{c in COMPETITOR, k in RESPONDENT}:
BETA[c,k] = T*Q[c,k] 
	+(sum{i in ATTRIBUTE, j in ATTRIBUTE_LEVEL[i]} (P_P[c,k,i,j]+P_M[c,k,i,j]))
	-(sum{n in CONSTRAINT, i in ATTRIBUTE, j in ATTRIBUTE_LEVEL[i]} S[n,i,j]*utility[k,i,j]*R[n,c,k])
;



subject to Z_P{c in COMPETITOR, k in RESPONDENT, i in ATTRIBUTE, j in ATTRIBUTE_LEVEL[i]}:
Q[c,k] 
+P_P[c,k,i,j] 
+(sum{n in CONSTRAINT} S[n,i,j]*utility_uncertainty[k,i,j]*R[n,c,k])
>=
utility_uncertainty[k,i,j]*(X[i,j]-X_COMP[c,i,j])
;


subject to Z_M{c in COMPETITOR, k in RESPONDENT, i in ATTRIBUTE, j in ATTRIBUTE_LEVEL[i]}:
Q[c,k] 
+P_M[c,k,i,j] 
-(sum{n in CONSTRAINT} S[n,i,j]*utility_uncertainty[k,i,j]*R[n,c,k])
>=
-utility_uncertainty[k,i,j]*(X[i,j]-X_COMP[c,i,j])
;







/*
subject to force1 :
	X['location','mountain'] = 1
;


subject to force2 :
	X['breakfast','american'] = 1
;

subject to force3 :
	X['style','casual'] = 1
;


subject to force4 :
	X['room','suite'] = 1
;

*/


